#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/*
---------------------------------------------------------------------------------
CONVERT INSTANCES:

++++++++++++  FROM FORMAT:

see: http://www.di.unipi.it/optimize/Data/MMCF.html#Plnr

A single file containing:

 Number of nodes
 Number of arcs
 Number ofcommodities (OD-pairs)

For each arc:
 startnode endnode cost capacity

For each OD-pair
 origin destination demand


++++++++++++  TO FORMAT:

Four-files mnetgen format, an extension of the original file
format produced by mnetgen (see mnetgen.C).

see: http://www.di.unipi.it/optimize/Data/MMCF.html#MNetGen

++++++++++++  USAGE:

 ./ly2mnet [instance name]
-------------------------------------------------------------------------------
*/



#define MAXCHARN 256
#define TOLERANCE       0.0000001

typedef struct info{						//information about the instance
            int commn;				//number of commodities
            int nodesn;				//number of nodes
            int arcsn;				//number of arcs
            int mutual;				//number of mutual capacities
        } Info;

typedef struct arc							//represents an arc
        {
            int name;				//name of the arc (maybe usefull...maybe not!)
            int fromnode;			//name of starting node
            int tonode;				//name of arriving node
            int commodity;			//commodity refferred to the arc
            double cost;            //cost of arc
            double cap;				//capacity
            int mcap;				//mutual capacity
        } Arc;

typedef struct node							//represents a node
        {
            int name;				//name of the node (maybe usefull...maybe not!)
            int commodity;			//commodity
            double supply;			//supply demand/forniture
        } Node;



void ly2mnet(char *file);



int main(int argc, char **argv)
{

    if(argc < 2){
        printf("\nUsage: ./ly2mnet [instance name]\n");
    }
    ly2mnet(argv[1]);

    printf("\nInstance %s coverted\n",argv[1]);
    return 0;
}



void ly2mnet(char *file){


int i,a,n,k = 0;    //counters
FILE *inFile;       //input file pointer

Info info;
Arc *arcs;
Node *nodes;

//Temp vars
int t_from = 0;
int t_to = 0;
int t_cost = 0;
int t_mcap = 0;
int t_odorigin = 0;
int t_oddest = 0;
int t_odsupply = 0;
double *mutuals;
//ixofcommnode[k][n] stores the index in nodes[] related with comm k and node n
int **ixofcommnode;



/*
-------------------------------------------------------------------------------
--------------------- Read instance in ly format ------------------------------

Format of the "planar" networks:

Number of nodes
Number of arcs
Number ofcommodities (OD-pairs)

For each arc:
 startnode endnode cost capacity

For each OD-pair
 origin destination demand
-------------------------------------------------------------------------------
*/


inFile = fopen(file,"r");

fscanf(inFile, "%d", &info.nodesn);
fscanf(inFile, "%d", &info.arcsn);
fscanf(inFile, "%d", &info.commn);

info.mutual = info.arcsn;

nodes = (Node *)calloc(info.nodesn*info.commn,sizeof(Node));
arcs = (Arc *)calloc(info.arcsn,sizeof(Arc));
mutuals = (double *)calloc(info.mutual+1,sizeof(double));


mutuals[0] = -1;
i = 0;
for(a = 0; a < info.arcsn;a++){
    fscanf(inFile, "%d\t%d\t%d\t%d", &t_from,&t_to,&t_cost,&t_mcap);
    mutuals[a+1] = t_mcap;
    arcs[a].name = a;
    arcs[a].fromnode = t_from;
    arcs[a].tonode = t_to;
    arcs[a].commodity = -1;
    arcs[a].cost = t_cost;
    arcs[a].cap = (double) t_mcap;
    arcs[a].mcap = a+1;
}


ixofcommnode = (int **) calloc(info.commn+1,sizeof(int *));
for(k = 0;k<= info.commn;k++)
    ixofcommnode[k] = (int *) calloc(info.nodesn+1,sizeof(int));

i = 0;
for(k = 1; k <= info.commn;k++){
    for(n = 1; n <= info.nodesn;n++){
        nodes[i].name = n;
        nodes[i].commodity = k;
        nodes[i].supply = 0;
        ixofcommnode[k][n] = i;
        i++;
    }
}

for(k = 1; k <= info.commn;k++){
    fscanf(inFile, "%d\t%d\t%d", &t_odorigin,&t_oddest,&t_odsupply);
    nodes[ixofcommnode[k][t_odorigin]].supply = t_odsupply;
    nodes[ixofcommnode[k][t_oddest]].supply = -t_odsupply;
}

fclose(inFile);


for(k = 0; k <= info.commn;k++){ free(ixofcommnode[k]); }
free(ixofcommnode);

/*
-------------------------------------------------------------------------------
--------------------- Write instance in mnet format ---------------------------
-------------------------------------------------------------------------------
*/


//Write file .mut
char outfilename[MAXCHARN];
char mutfilename[MAXCHARN];
FILE *mutfile;

strcpy(outfilename,file);

strcpy(mutfilename,outfilename);
strcat(mutfilename,".mut");


mutfile = fopen(mutfilename,"w");

for(i = 1; i <= info.mutual;i++ )
    fprintf(mutfile,"%d\t%f\n",i, mutuals[i]);

fclose(mutfile);



//Write file .arc
char arcfilename[MAXCHARN];
FILE *arcfile;


strcpy(arcfilename,outfilename);
strcat(arcfilename,".arc");


arcfile = fopen(arcfilename,"w");

for(a = 0; a < info.arcsn;a++ ){
    fprintf(arcfile,"%d\t%d\t%d\t%d\t%f\t%f\t%d\n",
            arcs[a].name+1,
            arcs[a].fromnode,
            arcs[a].tonode,
            -1,
            arcs[a].cost,
            arcs[a].cap,
            arcs[a].mcap);
}


fclose(arcfile);



//Write file .sup
char supfilename[MAXCHARN];
FILE *supfile;

strcpy(supfilename,outfilename);
strcat(supfilename,".sup");


supfile = fopen(supfilename,"w");

for(i = 0; i < info.nodesn*info.commn;i++ ){
    if(nodes[i].supply > 0 + TOLERANCE || nodes[i].supply < 0 - TOLERANCE )
        fprintf(supfile,"%d\t%d\t%f\n",
                nodes[i].name,
                nodes[i].commodity,
                nodes[i].supply);
}


fclose(supfile);




//Write file .nod
char nodfilename[MAXCHARN];
FILE *nodfile;

strcpy(nodfilename,outfilename);
strcat(nodfilename,".nod");


nodfile = fopen(nodfilename,"w");



fprintf(supfile,"%d\n",info.commn);
fprintf(supfile,"%d\n",info.nodesn);
fprintf(supfile,"%d\n",info.arcsn);
fprintf(supfile,"%d\n",info.mutual);


fclose(nodfile);


free(arcs);
free(nodes);
free(mutuals);

}

